@@ -1,4 +1,4 @@
-// $Id: MiscControlPage.cpp,v 1.43 2005/05/14 11:17:13 h_oudejans Exp $
+// $Id: MiscControlPage.cpp 9266 2009-02-26 23:36:58Z mthuurne $
 // MiscControlPage.cpp: implementation of the MiscControlPage class.
 //
 //////////////////////////////////////////////////////////////////////
@@ -60,7 +60,7 @@ MiscControlPage::MiscControlPage(wxWindow * parent, openMSXController * controll
 	wxXmlResource::Get()->LoadPanel(this, parent, wxT("MiscControlPage"));
 	m_controller = controller;
 	m_powerButton = (wxToggleButton *)FindWindowByName(wxT("PowerButton"));
-	m_pauseButton = (wxToggleButton *)FindWindowByName(wxT("PauseButton"));	
+	m_pauseButton = (wxToggleButton *)FindWindowByName(wxT("PauseButton"));
 	m_resetButton = (wxButton *)FindWindowByName(wxT("ResetButton"));
 	m_firmwareButton = (wxToggleButton *)FindWindowByName(wxT("FirmwareButton"));
 	m_speedIndicator = (wxTextCtrl *)FindWindowByName(wxT("SpeedIndicator"));
@@ -78,7 +78,7 @@ MiscControlPage::MiscControlPage(wxWindow * parent, openMSXController * controll
 
 	m_defaultMaxFrameSkipButton = (wxButton *)FindWindowByName (wxT("DefaultMaxFrameSkipButton"));
 	m_defaultMinFrameSkipButton = (wxButton *)FindWindowByName (wxT("DefaultMinFrameSkipButton"));
-	
+
 	m_speedSlider->SetTickFreq (25,1);
 	m_maxFrameSkipSlider->SetTickFreq (5,1);
 	m_minFrameSkipSlider->SetTickFreq (5,1);
@@ -90,13 +90,48 @@ MiscControlPage::MiscControlPage(wxWindow * parent, openMSXController * controll
 	m_frameskipLabel = (wxStaticText *)FindWindowByName(wxT("FrameSkipLabel"));
 	m_frameskipMaxLabel = (wxStaticText *)FindWindowByName(wxT("FrameSkipMaxLabel"));
 	m_frameskipMinLabel = (wxStaticText *)FindWindowByName(wxT("FrameSkipMinLabel"));
-	m_emulationSpeedLabel = (wxStaticText *)FindWindowByName(wxT("EmulationSpeedLabel"));	
-	
-// temporary hardcoded joystick port devices (not for BSD)
+	m_emulationSpeedLabel = (wxStaticText *)FindWindowByName(wxT("EmulationSpeedLabel"));
+
+
+
+	wxComboBox* box = (wxComboBox *)FindWindowByName (wxT("PrinterportSelector"));
+	box->Clear();
+	box->Append(wxT("--empty--"));
+	box->Append(wxT("logger"));
+	box->Append(wxT("simpl"));
+	box->Append(wxT("msx-printer"));
+	box->Append(wxT("epson-printer"));
+
+	/* This doesn't do anything, because it is "disabled" according to a comment in ConfigurationData
+	wxString filename;
+	wxTextCtrl * text = (wxTextCtrl *)FindWindowByName(wxT("PrinterLogFile"));
+	wxString current;
+	int pos;
+	ConfigurationData::instance()->GetParameter(ConfigurationData::CD_PRINTERPORT,current);
+	ConfigurationData::instance()->GetParameter(ConfigurationData::CD_PRINTERFILE,filename);
+	text->SetValue(filename);
+
+	pos = box->FindString(current);
+	if (pos != -1){
+			box->SetSelection(pos);
+		}
+		else{
+			box->SetSelection(0);
+		}
+	OnPrinterportChanged(false); */
+}
+
+MiscControlPage::~MiscControlPage()
+{
+}
+
+void MiscControlPage::FillInitialJoystickPortValues ()
+{
+//  temporary hardcoded joystick port devices (not for BSD)
 	wxString current;
 	int pos;
 
-#ifndef __BSD__ 
+#ifndef __BSD__
 	wxJoystick joy(wxJOYSTICK1);
 	wxString temp;
 #endif
@@ -108,7 +143,12 @@ MiscControlPage::MiscControlPage(wxWindow * parent, openMSXController * controll
 		box[i]->Clear();
 		box[i]->Append(wxT("--empty--"));
 		box[i]->Append(wxT("mouse"));
-		box[i]->Append(wxT("keyjoystick"));
+		box[i]->Append(wxT("joynet"));
+		box[i]->Append(wxT("tetris2-protection"));
+		box[i]->Append(wxT("magic-key"));
+		box[i]->Append(wxT("keyjoystick1"));
+		box[i]->Append(wxT("keyjoystick2"));
+
 #ifndef __BSD__
 		for (int j=1;j<=joy.GetNumberJoysticks();j++) {
 			temp.sprintf(wxT("joystick%d"),j);
@@ -119,40 +159,13 @@ MiscControlPage::MiscControlPage(wxWindow * parent, openMSXController * controll
 		pos = box[i]->FindString(current);
 		if (pos != -1){
 			box[i]->SetSelection(pos);
-		}		
-		else{	
+		}
+		else{
 			box[i]->SetSelection(0);
-		}	
+		}
 	}
 	m_oldJoy1 = box[0]->GetValue();
 	m_oldJoy2 = box[1]->GetValue();
-
-	
-	
-	box[0] = (wxComboBox *)FindWindowByName (wxT("PrinterportSelector"));
-	box[0]->Clear();
-	box[0]->Append(wxT("--empty--"));
-	box[0]->Append(wxT("logger"));
-	box[0]->Append(wxT("simpl"));	
-
-	wxString filename;
-	wxTextCtrl * text = (wxTextCtrl *)FindWindowByName(wxT("PrinterLogFile"));
-	ConfigurationData::instance()->GetParameter(ConfigurationData::CD_PRINTERPORT,current);
-	ConfigurationData::instance()->GetParameter(ConfigurationData::CD_PRINTERFILE,filename);
-	text->SetValue(filename);
-		
-	pos = box[0]->FindString(current);
-	if (pos != -1){
-			box[0]->SetSelection(pos);
-		}		
-		else{	
-			box[0]->SetSelection(0);
-		}
-	OnPrinterportChanged(false);
-}
-
-MiscControlPage::~MiscControlPage()
-{
 }
 
 void MiscControlPage::OnReset(wxCommandEvent &event)
@@ -245,7 +258,7 @@ void MiscControlPage::OnSetDefaultMinFrameSkip(wxCommandEvent &event)
 
 void MiscControlPage::OnSetThrottle(wxCommandEvent &event)
 {
-	wxToggleButton * button = (wxToggleButton *)event.m_eventObject;
+	wxToggleButton * button = (wxToggleButton *)event.GetEventObject();
 	if (button->GetValue())
 	{
 		m_controller->WriteCommand (wxT("set throttle on"));
@@ -255,12 +268,12 @@ void MiscControlPage::OnSetThrottle(wxCommandEvent &event)
 	{
 		m_controller->WriteCommand (wxT("set throttle off"));
 		button->SetLabel(wxT("Off"));
-	}	
+	}
 }
 
 void MiscControlPage::OnSetCmdTiming(wxCommandEvent &event)
 {
-	wxToggleButton * button = (wxToggleButton *)event.m_eventObject;
+	wxToggleButton * button = (wxToggleButton *)event.GetEventObject();
 	if (button->GetValue())
 	{
 		m_controller->WriteCommand (wxT("set cmdtiming broken"));
@@ -300,7 +313,7 @@ void MiscControlPage::SetControlsOnLaunch()
 	m_frameskipLabel->Enable(true);
 	m_frameskipMaxLabel->Enable(true);
 	m_frameskipMinLabel->Enable(true);
-	m_emulationSpeedLabel->Enable(true);	
+	m_emulationSpeedLabel->Enable(true);
 	wxWindow * temp = FindWindowByLabel(wxT("Performance Controls"));
 	if (temp){
 		temp->Enable(true);
@@ -370,7 +383,7 @@ void MiscControlPage::OnInputSpeed(wxCommandEvent &event)
 	{
 		unsigned long num;
 		text.ToULong(&num);
-		if (num >500) 
+		if (num >500)
 		{
 			num = 500;
 			text = wxT("500");
@@ -402,7 +415,7 @@ void MiscControlPage::OnInputMinFrameskip(wxCommandEvent &event)
 			m_controller->WriteCommand (wxString(wxT("set minframeskip ") + text));
 			m_minFrameSkipSlider->SetValue(num);
 		}
-	}	
+	}
 }
 
 void MiscControlPage::OnInputMaxFrameskip(wxCommandEvent &event)
@@ -423,7 +436,7 @@ void MiscControlPage::OnInputMaxFrameskip(wxCommandEvent &event)
 			m_controller->WriteCommand (wxString(wxT("set maxframeskip ") + text));
 			m_maxFrameSkipSlider->SetValue(num);
 		}
-	}	
+	}
 }
 
 void MiscControlPage::SetSpeed (wxString value)
@@ -531,7 +544,7 @@ void MiscControlPage::OnJoystickChanged()
 		box2 = (wxComboBox *)FindWindowByName (wxT("Joyport2Selector"));
 		oldValue1 = &m_oldJoy1;
 		oldValue2 = &m_oldJoy2;
-	} 
+	}
 	else if (box->GetName() == wxT("Joyport2Selector")) {
 		connector = wxT("joyportb ");
 		connector2 = wxT("joyporta");
@@ -539,7 +552,7 @@ void MiscControlPage::OnJoystickChanged()
 		oldValue1 = &m_oldJoy2;
 		oldValue2 = &m_oldJoy1;
 	}
-	
+
 	if (box->GetValue() != wxT("--empty--")) {
 		value = box->GetValue();
 		cmd = wxT("plug ");
@@ -567,7 +580,7 @@ This device will then be removed from any other port(s)."),wxT("Warning"),wxOK |
 		*oldValue1 = box->GetValue();
 	}
 	wxComboBox * joy1 = (wxComboBox *)FindWindowByName (wxT("Joyport1Selector"));
-	wxComboBox * joy2 = (wxComboBox *)FindWindowByName (wxT("Joyport2Selector"));	
+	wxComboBox * joy2 = (wxComboBox *)FindWindowByName (wxT("Joyport2Selector"));
 	ConfigurationData::instance()->SetParameter(ConfigurationData::CD_JOYPORT1,joy1->GetValue());
 	ConfigurationData::instance()->SetParameter(ConfigurationData::CD_JOYPORT2,joy2->GetValue());
 	ConfigurationData::instance()->SaveData();
@@ -578,8 +591,8 @@ void MiscControlPage::OnChangeRenShaTurbo(wxScrollEvent & event)
 {
 	wxString value;
 	value.sprintf (wxT("%ld"),event.GetInt());
-	
-	
+
+
 	m_controller->WriteCommand(wxString (wxT("set renshaturbo ")) + value);
 }
 
@@ -603,7 +616,7 @@ void MiscControlPage::OnPrinterportChanged(bool save)
 	if (m_controller->IsOpenMSXRunning()){
 		m_controller->UpdateMixer();
 	}
-		
+
 }
 
 void MiscControlPage::OnChangePrinterLogFile(wxCommandEvent &event)
