@@ -1,4 +1,4 @@
-// $Id: openMSXController.cpp,v 1.88 2005/05/29 12:18:24 h_oudejans Exp $
+// $Id: openMSXController.cpp 9649 2009-04-28 04:14:53Z mfeingol $
 // openMSXController.cpp: implementation of the openMSXController class.
 //
 //////////////////////////////////////////////////////////////////////
@@ -22,12 +22,15 @@
 #include "ScreenShotDlg.h"
 #include <cassert>
 
+
 #if wxCHECK_VERSION(2,5,0)
 #define WX24SIGNCAST
 #else
 #define WX24SIGNCAST (unsigned int)
 #endif
 
+
+
 //////////////////////////////////////////////////////////////////////
 // Construction/Destruction
 //////////////////////////////////////////////////////////////////////
@@ -40,7 +43,6 @@ openMSXController::openMSXController(wxWindow * target)
 	m_pluggables.Clear();
 	m_connectors.Clear();
 	InitLaunchScript ();
-	m_socket = NULL;
 }
 
 openMSXController::~openMSXController()
@@ -194,7 +196,12 @@ void openMSXController::HandleParsedOutput(wxCommandEvent &event)
 				m_appWindow->UpdateLed (data->name, data->contents);
 			}
 			if (data->updateType == CatapultXMLParser::UPDATE_STATE) {
-				m_appWindow->UpdateState (data->name, data->contents);
+				if (data->name == wxT("cassetteplayer")){
+					m_appWindow->m_sessionPage->SetCassetteMode(data->contents);
+				}
+				else{
+					m_appWindow->UpdateState (data->name, data->contents);
+				}
 			}
 			if (data->updateType == CatapultXMLParser::UPDATE_SETTING) {
 				wxString lastcmd = PeekPendingCommand();
@@ -227,7 +234,8 @@ void openMSXController::HandleParsedOutput(wxCommandEvent &event)
 					eject = true;
 				}
 				if ((lastcmd.Mid(0,data->name.Len()+1) != wxString(data->name + wxT(" "))) ||
-					(!eject && (lastcmd.Mid(space+1) != (wxString (wxT("\"")) + data->contents + wxString(wxT("\"")))))){
+					(!eject && (lastcmd.Mid(space+1) != (wxString (wxT("\"")) + data->contents + wxString(wxT("\""))))) ||
+					(lastcmd.Left(18) == wxT("cassetteplayer new"))){
 						m_appWindow->m_videoControlPage->UpdateSetting (data->name, data->contents);
 						m_appWindow->m_sessionPage->UpdateSessionData();
 				}
@@ -241,7 +249,7 @@ void openMSXController::HandleParsedOutput(wxCommandEvent &event)
 					}
 					else{
 						wxString command = GetPendingCommand();
-						if (command == GetInfoCommand(wxT("fps"))) {
+						if (command == wxT("openmsx_info fps")) {
 							m_appWindow->SetFPSdisplay(data->contents);
 						}
 						else if (command == wxT("save_settings")){
@@ -325,21 +333,27 @@ bool openMSXController::WriteCommand(wxString msg, TargetType target)
 {
 	if (!m_openMsxRunning)
 		return false;
+
 	CommandEntry temp;
 	temp.command = msg;
 	temp.target = target;
 	m_commands.push_back(temp);
-	char * convertBuffer = new char[msg.Len()+1];
-	strcpy(convertBuffer,(const char*) (wxConvUTF8.cWX2MB(msg)));
-	xmlChar* buffer = xmlEncodeEntitiesReentrant(NULL,(const xmlChar *)convertBuffer);
+
+	xmlChar* buffer = xmlEncodeEntitiesReentrant(NULL,(const xmlChar *)(const char *)(wxConvUTF8.cWX2MB(msg)));
+	if (!buffer)
+		return false;
+
+	bool result = false;
 	char * commandBuffer = new char[strlen((const char *) buffer)+25];
-	strcpy (commandBuffer,"<command>");
-	strcat (commandBuffer,(const char *)buffer);
-	strcat (commandBuffer,"</command>\n");
-	bool result = WriteMessage((xmlChar *)commandBuffer,strlen(commandBuffer));
-	if (buffer != NULL) {
-		xmlFree(buffer);
+	if (commandBuffer) {
+		strcpy (commandBuffer,"<command>");
+		strcat (commandBuffer,(const char *)buffer);
+		strcat (commandBuffer,"</command>\n");
+		result = WriteMessage((xmlChar *)commandBuffer,strlen(commandBuffer));
+		delete [] commandBuffer;
 	}
+
+	xmlFree(buffer);
 	return result;
 }
 
@@ -349,7 +363,7 @@ wxString openMSXController::GetPendingCommand()
 	CommandEntry entry;
 	wxString pending;
 	if (m_commands.empty()){	// TODO: why is assert (!m_commands.empty()) triggered ?
-		pending = wxT("");			// it can only happen if a reply is received without a previous send command
+		pending = wxT("");			// it can only happen if a reply is received without a previously sent command
 	}
 	else{
 		entry = m_commands.front();
@@ -387,13 +401,14 @@ enum openMSXController::TargetType openMSXController::PeekPendingCommandTarget()
 	return pending;
 }
 
-void openMSXController::StartOpenMSX(wxString cmd, bool getversion)
+bool openMSXController::StartOpenMSX(wxString cmd, bool getversion)
 {
 	m_openMSXID++;
+	bool retval = true;
 	if (getversion) {
 		m_appWindow->SetStatusText(wxT("Initializing..."));
 		wxString versioninfo = GetOpenMSXVersionInfo(cmd);
-		SetupOpenMSXParameters(versioninfo);
+		retval = SetupOpenMSXParameters(versioninfo);
 		m_appWindow->SetStatusText(wxT("Ready"));
 	}
 	else {
@@ -401,6 +416,7 @@ void openMSXController::StartOpenMSX(wxString cmd, bool getversion)
 		m_appWindow->EnableSaveSettings(true);
 		Launch(cmd);
 	}
+	return retval;
 }
 
 void openMSXController::HandleNormalLaunchReply(wxCommandEvent &event)
@@ -408,10 +424,6 @@ void openMSXController::HandleNormalLaunchReply(wxCommandEvent &event)
 	executeLaunch(&event);
 }
 
-wxString openMSXController::GetInfoCommand (wxString parameter)
-{
-	return wxString (wxT("join [openmsx_info ") + parameter +wxT("] \\n"));
-}
 
 int openMSXController::InitConnectors(wxString dummy, wxString connectors)
 {
@@ -554,21 +566,16 @@ bool openMSXController::SetupOpenMSXParameters(wxString version)
 			}
 		}
 	}
+	// printf ("Detected openMSX version: %d\n", ver);
 	if (ver == -1) {
-		wxMessageBox (wxT("Unable to determine openMSX version!\nPlease upgrade to 0.5.0 or higher."),wxT("Error"));
+		wxMessageBox (wxT("Unable to determine openMSX version!\nPlease upgrade to 0.6.2 or higher.\n(Or contact the authors.)"),wxT("Error"));
 		return false;
 	}
-	if (ver < 500) {
-		wxMessageBox (wxT("The openMSX version you are using is too old!\nPlease upgrade to 0.5.0 or higher."),wxT("Error"));
+	if (ver < 602) {
+		wxMessageBox (wxT("The openMSX version you are using is too old!\nPlease upgrade to 0.6.2 or higher."),wxT("Error"));
 		return false;
 	}
-	if ((version.Find(wxT("-dev")) != -1) || (ver > 500)) {
-		// put specific code for NEW versions here
-
-	}
-	else{
-		// put specific code for OLD supported versions here
-	}
+	m_appWindow->m_miscControlPage->FillInitialJoystickPortValues();
 	m_appWindow->m_launch_AbortButton->Enable(true);
 	return true;
 }
@@ -582,14 +589,16 @@ void openMSXController::InitLaunchScript ()
 	AddLaunchInstruction (wxT("set power on"),wxT("e"),wxT("power"),&openMSXController::UpdateSetting,true);
 	AddLaunchInstruction (wxT("unset renderer"),wxT("e"),wxT(""),NULL,true);
 	AddLaunchInstruction (wxT("@execute"),wxT(""),wxT(""),&openMSXController::EnableMainWindow,false);
-	AddLaunchInstruction (wxT("#info renderer"),wxT(""),wxT("RendererSelector"),&openMSXController::FillComboBox,true);
-	AddLaunchInstruction (wxT("#info scaler"),wxT(""),wxT("ScalerSelector"),&openMSXController::FillComboBox,true);
-	AddLaunchInstruction (wxT("#info accuracy"),wxT(""),wxT("AccuracySelector"),&openMSXController::FillComboBox,false);
+	AddLaunchInstruction (wxT("^info renderer"),wxT(""),wxT("RendererSelector"),&openMSXController::FillComboBox,true);
+	AddLaunchInstruction (wxT("^info scale_algorithm"),wxT(""),wxT("ScalerAlgoSelector"),&openMSXController::FillComboBox,true);
+	AddLaunchInstruction (wxT("lindex [openmsx_info setting scale_factor] 2"),wxT(""),wxT("ScalerFactorSelector"),&openMSXController::FillRangeComboBox,true);
+	AddLaunchInstruction (wxT("^info accuracy"),wxT(""),wxT("AccuracySelector"),&openMSXController::FillComboBox,false);
 	AddLaunchInstruction (wxT("update enable media"),wxT(""),wxT(""),NULL,false);
 	AddLaunchInstruction (wxT("info exist frontswitch"),wxT(""),wxT("#"),&openMSXController::EnableFirmware,false);
 	AddLaunchInstruction (wxT("info exist firmwareswitch"),wxT(""),wxT("#"),&openMSXController::EnableFirmware,false);
 	AddLaunchInstruction (wxT("set renderer"),wxT(""),wxT("renderer"),&openMSXController::UpdateSetting,true);
-	AddLaunchInstruction (wxT("set scaler"),wxT(""),wxT("scaler"),&openMSXController::UpdateSetting,true);
+	AddLaunchInstruction (wxT("set scale_algorithm"),wxT(""),wxT("scale_algorithm"),&openMSXController::UpdateSetting,true);
+	AddLaunchInstruction (wxT("set scale_factor"),wxT(""),wxT("scale_factor"),&openMSXController::UpdateSetting,true);
 	AddLaunchInstruction (wxT("set accuracy"),wxT(""),wxT("accuracy"),&openMSXController::UpdateSetting,true);
 	AddLaunchInstruction (wxT("set deinterlace"),wxT(""),wxT("deinterlace"),&openMSXController::UpdateSetting,true);
 	AddLaunchInstruction (wxT("set limitsprites"),wxT(""),wxT("limitsprites"),&openMSXController::UpdateSetting,true);
@@ -608,11 +617,11 @@ void openMSXController::InitLaunchScript ()
 	AddLaunchInstruction (wxT("set minframeskip"),wxT(""),wxT("minframeskip"),&openMSXController::UpdateSetting,true);
 	AddLaunchInstruction (wxT("set throttle"),wxT(""),wxT("throttle"),&openMSXController::UpdateSetting,true);
 	AddLaunchInstruction (wxT("set cmdtiming"),wxT(""),wxT("cmdtiming"),&openMSXController::UpdateSetting,true);
-	AddLaunchInstruction (wxT("#info pluggable"),wxT("13"),wxT(""),&openMSXController::InitPluggables,true);
-	AddLaunchInstruction (wxT("#info_nostore pluggable *"),wxT(""),wxT("*"),&openMSXController::AddPluggableDescription,true);
-	AddLaunchInstruction (wxT("#info_nostore connectionclass *"),wxT(""),wxT("*"),&openMSXController::AddPluggableClass,false);
-	AddLaunchInstruction (wxT("#info connector"),wxT("10"),wxT(""),&openMSXController::InitConnectors,true);
-	AddLaunchInstruction (wxT("#info_nostore connectionclass *"),wxT(""),wxT("*"),&openMSXController::AddConnectorClass,false);
+	AddLaunchInstruction (wxT("!info pluggable"),wxT("13"),wxT(""),&openMSXController::InitPluggables,true);
+	AddLaunchInstruction (wxT("!info_nostore pluggable *"),wxT(""),wxT("*"),&openMSXController::AddPluggableDescription,true);
+	AddLaunchInstruction (wxT("!info_nostore connectionclass *"),wxT(""),wxT("*"),&openMSXController::AddPluggableClass,false);
+	AddLaunchInstruction (wxT("!info connector"),wxT("10"),wxT(""),&openMSXController::InitConnectors,true);
+	AddLaunchInstruction (wxT("!info_nostore connectionclass *"),wxT(""),wxT("*"),&openMSXController::AddConnectorClass,false);
 	AddLaunchInstruction (wxT("plug *"),wxT(""),wxT("*"),&openMSXController::AddConnectorContents,true);
 	AddLaunchInstruction (wxT("@checkfor msx-midi-in"),wxT("1"),wxT(""),NULL,false);
 	AddLaunchInstruction (wxT("set midi-in-readfilename"),wxT(""),wxT("midi-in-readfilename"),&openMSXController::UpdateSetting,true);
@@ -622,18 +631,19 @@ void openMSXController::InitLaunchScript ()
 	AddLaunchInstruction (wxT("set audio-inputfilename"),wxT(""),wxT("audio-inputfilename"),&openMSXController::UpdateSetting,true);
 	AddLaunchInstruction (wxT("@execute"),wxT(""),wxT(""),&openMSXController::InitConnectorPanel,false);
 	AddLaunchInstruction (wxT("@execute"),wxT(""),wxT(""),&openMSXController::InitAudioConnectorPanel,false);
-	AddLaunchInstruction (wxT("#info sounddevice"),wxT("5"),wxT(""),&openMSXController::InitSoundDevices,true);
-	AddLaunchInstruction (wxT("#info_nostore sounddevice *"),wxT(""),wxT("*"),&openMSXController::SetChannelType,true);
+//	AddLaunchInstruction (wxT("#info romtype"),wxT(""),wxT(""),&openMSXController::InitRomTypes,true);
+//	AddLaunchInstruction (wxT("#info_nostore romtype *"),wxT(""),wxT("*"),&openMSXController::SetRomDescription,true);
+	AddLaunchInstruction (wxT("!info sounddevice"),wxT("5"),wxT(""),&openMSXController::InitSoundDevices,true);
+	AddLaunchInstruction (wxT("!info_nostore sounddevice *"),wxT(""),wxT("*"),&openMSXController::SetChannelType,true);
 	AddLaunchInstruction (wxT("set master_volume"),wxT(""),wxT("master_volume"),&openMSXController::UpdateSetting,false);
 	AddLaunchInstruction (wxT("set *_volume"),wxT(""),wxT("*_volume"),&openMSXController::UpdateSetting,true);
-	AddLaunchInstruction (wxT("set *_mode"),wxT(""),wxT("*_mode"),&openMSXController::UpdateSetting,true);
+	AddLaunchInstruction (wxT("set *_balance"),wxT(""),wxT("*_balance"),&openMSXController::UpdateSetting,true);
 	AddLaunchInstruction (wxT("set mute"),wxT(""),wxT("mute"),&openMSXController::UpdateSetting,true);
 	AddLaunchInstruction (wxT("plug cassetteport"),wxT(""),wxT("cassetteport"),&openMSXController::EnableCassettePort,false);
+	AddLaunchInstruction (wxT("join [cassetteplayer] \\n"),wxT(""),wxT(""),&openMSXController::SetCassetteMode,false);
 	AddLaunchInstruction (wxT("update enable plug"),wxT(""),wxT(""),NULL,false);
 	AddLaunchInstruction (wxT("update enable unplug"),wxT(""),wxT(""),NULL,false);
 	AddLaunchInstruction (wxT("update enable status"),wxT(""),wxT(""),NULL,false);
-
-
 }
 
 void openMSXController::AddLaunchInstruction (wxString cmd, wxString action,
@@ -688,6 +698,12 @@ void openMSXController::executeLaunch (wxCommandEvent * event, int startLine)
 			if (tokens[0] == wxT("#info")) {
 				lastdata = data->contents;
 			}
+			if (tokens[0] == wxT("^info")) {
+				lastdata = data->contents;
+			}
+			if (tokens[0] == wxT("!info")) {
+				lastdata = data->contents;
+			}
 			HandleLaunchReply (cmd,event,m_launchScript[recvStep],&sendStep,recvLoop,lastdata);
 			if ((data->replyState == CatapultXMLParser::REPLY_NOK) ||
 				((cmd.Mid(0,10) == wxT("info exist")) && (data->contents == wxT("0")))) {
@@ -862,7 +878,37 @@ wxString openMSXController::translate(wxArrayString tokens, int loop, wxString l
 					tokens.RemoveAt(token+1);
 				}
 				parameter.Trim(true);
-				tokens[token] = GetInfoCommand(parameter);
+				tokens[token] = wxString(wxT("join [openmsx_info ") + parameter + wxT("] \\n"));
+			}
+			else {
+				assert(false); // invalid command
+			}
+			break;
+		case '!':
+			if (tokens[token].Mid(0,5)== wxT("!info")) {
+				wxString parameter = wxT("");
+				while (token < (tokens.GetCount()-1)) {
+					parameter += tokens[token+1];
+					parameter += wxT(" ");
+					tokens.RemoveAt(token+1);
+				}
+				parameter.Trim(true);
+				tokens[token] = wxString(wxT("join [machine_info ") + parameter + wxT("] \\n"));
+			}
+			else {
+				assert(false); // invalid command
+			}
+			break;
+		case '^':
+			if (tokens[token].Mid(0,5)== wxT("^info")) {
+				wxString parameter = wxT("");
+				while (token < (tokens.GetCount()-1)) {
+					parameter += tokens[token+1];
+					parameter += wxT(" ");
+					tokens.RemoveAt(token+1);
+				}
+				parameter.Trim(true);
+				tokens[token] = wxString(wxT("join [lindex [openmsx_info setting ") + parameter + wxT("] 2] \\n"));
 			}
 			else {
 				assert(false); // invalid command
@@ -961,6 +1007,25 @@ int openMSXController::FillComboBox (wxString setting,wxString data)
 	return 0; // don't skip any lines in the startup script
 }
 
+int openMSXController::FillRangeComboBox (wxString setting, wxString data)
+{
+	long min;
+	long max;
+	wxString range = wxT("");
+	int pos = data.Find(' ');
+	if (pos != -1){
+		data.Left(pos).ToLong(&min);
+		data.Mid(pos+1).ToLong(&max);
+		for (int index=min;index<=max;index++){
+			range << index;
+			range += wxT("\n");
+		}
+	}
+
+	m_appWindow->m_videoControlPage->FillComboBox(setting,range); // Just use any instance of CatapultPage
+	return 0; // don't skip any lines in the startup script
+}
+
 int openMSXController::EnableFirmware (wxString cmd, wxString data)
 {
 	if ((data != wxT("0")) || (cmd.Mid(0,4) == wxT("set "))) {
@@ -988,6 +1053,23 @@ int openMSXController::EnableMainWindow(wxString dummy1, wxString dummy2)
 	return 0; // don't skip any lines in the startup script
 }
 
+int openMSXController::InitRomTypes (wxString dummy, wxString data)
+{
+	wxArrayString types;
+	tokenize (data, wxT("\n"),types);
+	for (unsigned int index=0;index < types.GetCount();index++)
+	{
+		m_appWindow->m_sessionPage->AddRomType (types[index]);
+	}
+	return 0;
+}
+
+int openMSXController::SetRomDescription(wxString name, wxString data)
+{
+	m_appWindow->m_sessionPage->SetRomTypeFullName(name,data);
+	return 0;
+}
+
 int openMSXController::InitSoundDevices (wxString dummy, wxString data)
 {
 	wxArrayString channels;
@@ -1071,6 +1153,14 @@ int openMSXController::EnableCassettePort (wxString dummy, wxString data)
 	return 0; // don't skip any lines in the startup script
 }
 
+int openMSXController::SetCassetteMode (wxString dummy, wxString data)
+{
+	wxArrayString arrayData;
+	int args = tokenize(data,wxT("\n"),arrayData);
+	m_appWindow->m_sessionPage->SetCassetteMode (arrayData[args-1]);
+	return 0;
+}
+
 void openMSXController::UpdateMixer()
 {
 	executeLaunch(NULL,43);
